#include "XmlWriter.h"

#include "XmlDocument.h"
#include "XmlAttribute.h"

std::string XmlWriter::toString(XmlElement* element, unsigned depth)
{
    const auto prefix = std::string(2*depth, ' ');

    auto content = prefix + "<" + element->getTagName();
    for (std::size_t idx=0; idx< element->getNumAttributes(); idx++)
    {
        auto attribute = element->getAttribute(idx);
        content += " " + attribute->getName() + "=\"" + attribute->getValue() + "\"";
    }

    const auto num_children = element->getNumChildren();
    if (num_children == 0 && element->getText().empty())
    {
        content += "/>\n";
        return content;
    }
    else
    {
        content += ">";
    }

    if (!element->getText().empty())
    {
        content += element->getText();
    }

    if (num_children>0)
    {
        content += "\n";
    }
    for (std::size_t idx=0; idx< element->getNumChildren(); idx++)
    {
        auto child = element->getChild(idx);
        content += toString(child, depth+1);
    }
    if (num_children>0)
    {
        content += prefix;
    }

    content += "</" + element->getTagName() + ">\n";
    return content;
}

std::string XmlWriter::toString(XmlDocument* document)
{
    std::string content;
    if (auto prolog = document->getProlog())
    {
        content += "<?xml";
        for (std::size_t idx=0; idx< prolog->getNumAttributes(); idx++)
        {
            auto attribute = prolog->getAttribute(idx);
            content += " " + attribute->getName() + "=\"" + attribute->getValue() + "\"";
        }
        content += "?>\n";
    }

    if (auto root = document->getRoot())
    {
        content += toString(root);
    }
    return content;
}
